@@ -512,7 +512,7 @@ calendar_control_sensitize_calendar_commands (BonoboControl *control, GnomeCalen
 	n_selected = enable ? g_list_length (list) : 0;
 
 	event = (ECalendarViewEvent *) list ? list->data : NULL;
-	if (event)
+	if (event && event->comp_data)
 		e_cal_is_read_only (event->comp_data->client, &selected_read_only, NULL);
 	else
 		selected_read_only = TRUE;
@@ -536,9 +536,6 @@ calendar_control_sensitize_calendar_commands (BonoboControl *control, GnomeCalen
 	bonobo_ui_component_set_prop (uic, "/commands/Paste", "sensitive",
 				      default_read_only ? "0" : "1",
 				      NULL);
-	bonobo_ui_component_set_prop (uic, "/commands/Delete", "sensitive",
-				      n_selected == 0 || selected_read_only ? "0" : "1",
-				      NULL);
 
 	/* occurrence-related menu items */
 	has_recurrences = FALSE;
@@ -550,6 +547,9 @@ calendar_control_sensitize_calendar_commands (BonoboControl *control, GnomeCalen
 		}
 	}
 
+	bonobo_ui_component_set_prop (uic, "/commands/Delete", "sensitive",
+				      n_selected == 0 || selected_read_only || has_recurrences ? "0" : "1",
+				      NULL);
 	bonobo_ui_component_set_prop (uic, "/commands/DeleteOccurrence", "sensitive",
 				      has_recurrences ? "1" : "0",
 				      NULL);
@@ -571,6 +571,8 @@ sensitize_taskpad_commands (GnomeCalendar *gcal, BonoboControl *control, gboolea
 {
 	BonoboUIComponent *uic;
 	int n_selected;
+	ECalendarTable *task_pad;
+	ECalModel *model;
 	ECal *e_cal;
 	gboolean read_only = TRUE;
 	
@@ -578,7 +580,10 @@ sensitize_taskpad_commands (GnomeCalendar *gcal, BonoboControl *control, gboolea
 	g_assert (uic != NULL);
 
 	n_selected = enable ? gnome_calendar_get_num_tasks_selected (gcal) : 0;
-	e_cal = gnome_calendar_get_task_pad_e_cal (gcal);
+	task_pad = gnome_calendar_get_task_pad (gcal);
+	model = e_calendar_table_get_model (task_pad);
+	e_cal = e_cal_model_get_default_client (model);
+	
 	if (e_cal)
 		e_cal_is_read_only (e_cal, &read_only, NULL);
 	else
