@@ -34,6 +34,7 @@
 #include <gtk/gtkvpaned.h>
 #include <gtkhtml/gtkhtml.h>
 #include <gdk/gdkkeysyms.h>
+#include <gconf/gconf-client.h>
 
 #include <libgnomeprintui/gnome-print-dialog.h>
 
@@ -44,14 +45,14 @@
 
 #include <e-util/e-dialog-utils.h>
 
-#include <camel/camel-mime-message.h>
+/*#include <camel/camel-mime-message.h>*/
 #include <camel/camel-stream.h>
-#include <camel/camel-stream-filter.h>
+/*#include <camel/camel-stream-filter.h>
 #include <camel/camel-mime-filter.h>
 #include <camel/camel-mime-filter-tohtml.h>
 #include <camel/camel-mime-filter-enriched.h>
 #include <camel/camel-multipart.h>
-#include <camel/camel-stream-mem.h>
+#include <camel/camel-stream-mem.h>*/
 #include <camel/camel-url.h>
 
 #include <bonobo/bonobo-main.h>
@@ -98,6 +99,10 @@ struct _EMFolderBrowserPrivate {
 	GalViewInstance *view_instance;
 	GalViewMenus *view_menus;
 
+	guint search_menu_activated_id;
+	guint search_activated_id;
+	guint search_query_changed_id;
+
 	guint vpane_resize_id;
 	guint list_built_id;	/* hook onto list-built for delayed 'select first unread' stuff */
 	
@@ -187,9 +192,9 @@ emfb_init(GObject *o)
 		e_search_bar_set_menu ((ESearchBar *)emfb->search, emfb_search_items);
 		gtk_widget_show((GtkWidget *)emfb->search);
 
-		g_signal_connect(emfb->search, "menu_activated", G_CALLBACK(emfb_search_menu_activated), emfb);
-		g_signal_connect(emfb->search, "search_activated", G_CALLBACK(emfb_search_search_activated), emfb);
-		g_signal_connect(emfb->search, "query_changed", G_CALLBACK(emfb_search_query_changed), emfb);
+		p->search_menu_activated_id = g_signal_connect(emfb->search, "menu_activated", G_CALLBACK(emfb_search_menu_activated), emfb);
+		p->search_activated_id = g_signal_connect(emfb->search, "search_activated", G_CALLBACK(emfb_search_search_activated), emfb);
+		p->search_query_changed_id = g_signal_connect(emfb->search, "query_changed", G_CALLBACK(emfb_search_query_changed), emfb);
 
 		gtk_box_pack_start((GtkBox *)emfb, (GtkWidget *)emfb->search, FALSE, TRUE, 0);
 	}
@@ -416,7 +421,7 @@ emfb_search_search_activated(ESearchBar *esb, EMFolderBrowser *emfb)
 	EMFolderView *emfv = (EMFolderView *) emfb;
 	char *search_word, *search_state;
 	
-	if (emfv->list == NULL)
+	if (emfv->list == NULL || emfv->folder == NULL)
 		return;
 	
 	g_object_get (esb, "query", &search_word, NULL);
@@ -562,7 +567,10 @@ emfb_mark_all_read(BonoboUIComponent *uid, void *data, const char *path)
 	EMFolderView *emfv = data;
 	GPtrArray *uids;
 	int i;
-	
+
+	if (emfv->folder == NULL)
+		return;
+
 	uids = camel_folder_get_uids(emfv->folder);
 	camel_folder_freeze(emfv->folder);
 	for (i=0;i<uids->len;i++)
@@ -709,7 +717,11 @@ static EPixmap emfb_pixmaps[] = {
 };
 
 static const EMFolderViewEnable emfb_enable_map[] = {
+	{ "EditInvertSelection", EM_POPUP_SELECT_FOLDER },
+	{ "EditSelectAll", EM_POPUP_SELECT_FOLDER },
 	{ "EditSelectThread", EM_FOLDER_VIEW_SELECT_THREADED },
+	{ "MailPost", EM_POPUP_SELECT_FOLDER },
+	{ "MessageMarkAllAsRead", EM_POPUP_SELECT_FOLDER },
 	{ "ViewHideSelected", EM_POPUP_SELECT_MANY },
 	{ "ViewShowAll", EM_FOLDER_VIEW_SELECT_HIDDEN },
 	{ NULL },
@@ -726,8 +738,10 @@ emfb_hide_deleted(BonoboUIComponent *uic, const char *path, Bonobo_UIComponent_E
 
 	gconf = mail_config_get_gconf_client ();
 	gconf_client_set_bool(gconf, "/apps/evolution/mail/display/show_deleted", state[0] == '0', NULL);
-	if (!(emfv->folder && (emfv->folder->folder_flags & CAMEL_FOLDER_IS_TRASH)))
+	if (!(emfv->folder && (emfv->folder->folder_flags & CAMEL_FOLDER_IS_TRASH))) {
 		message_list_set_hidedeleted(emfv->list, state[0] != '0');
+		emfv->hide_deleted = state[0] != '0';
+	}
 }
 
 static void
@@ -903,6 +917,7 @@ static void
 emfb_set_folder(EMFolderView *emfv, CamelFolder *folder, const char *uri)
 {
 	EMFolderBrowser *emfb = (EMFolderBrowser *) emfv;
+	struct _EMFolderBrowserPrivate *p = emfb->priv;
 
 	emfb_parent->set_folder(emfv, folder, uri);
 	
@@ -911,27 +926,46 @@ emfb_set_folder(EMFolderView *emfv, CamelFolder *folder, const char *uri)
 	   defaults */
 	if (folder) {
 		char *sstate;
+		int state;
+		GConfClient *gconf = mail_config_get_gconf_client();
+
+		/* FIXME: this mostly copied from activate() */
+		if ((sstate = camel_object_meta_get(folder, "evolution:show_preview"))) {
+			state = sstate[0] != '0';
+			g_free(sstate);
+		} else
+			state = gconf_client_get_bool(gconf, "/apps/evolution/mail/display/show_preview", NULL);
+		em_folder_browser_show_preview(emfb, state);
+		if (emfv->uic)
+			bonobo_ui_component_set_prop(emfv->uic, "/commands/ViewPreview", "state", state?"1":"0", NULL);
 		
-		if ((sstate = camel_object_meta_get (folder, "evolution:show_preview"))) {
-			em_folder_browser_show_preview (emfb, sstate[0] != '0');
-			g_free (sstate);
-		}
+		if ((sstate = camel_object_meta_get(folder, "evolution:thread_list"))) {
+			state = sstate[0] != '0';
+			g_free(sstate);
+		} else
+			state = gconf_client_get_bool(gconf, "/apps/evolution/mail/display/thread_list", NULL);
+		message_list_set_threaded(emfv->list, state);
+		if (emfv->uic)
+			bonobo_ui_component_set_prop(emfv->uic, "/commands/ViewThreaded", "state", state?"1":"0", NULL);
 		
-		if ((sstate = camel_object_meta_get (folder, "evolution:thread_list"))) {
-			message_list_set_threaded (emfv->list, sstate[0] != '0');
-			g_free (sstate);
-		}
-		
-		sstate = camel_object_meta_get (folder, "evolution:search_state");
-		g_object_set (emfb->search, "state", sstate, NULL);
-		g_free (sstate);
+		sstate = camel_object_meta_get(folder, "evolution:search_state");
+		g_object_set(emfb->search, "state", sstate, NULL);
+		g_free(sstate);
 		
+		/* set the query manually, so we dont pop up advanced or saved search stuff */
+		g_object_get(emfb->search, "query", &sstate, NULL);
+		message_list_set_search(emfb->view.list, sstate);
+		g_free(sstate);
+
 		if ((sstate = camel_object_meta_get (folder, "evolution:selected_uid")))
 			emfb->priv->select_uid = sstate;
+		else {
+			g_free(p->select_uid);
+			p->select_uid = NULL;
+		}
 		
 		if (emfv->list->cursor_uid == NULL && emfb->priv->list_built_id == 0)
-			emfb->priv->list_built_id =
-				g_signal_connect (emfv->list, "message_list_built", G_CALLBACK (emfb_list_built), emfv);
+			emfb->priv->list_built_id = g_signal_connect(emfv->list, "message_list_built", G_CALLBACK (emfb_list_built), emfv);
 		
 		/*emfb_create_view_instance (emfb, folder, uri);*/
 		if (emfv->uic)
@@ -1034,7 +1068,7 @@ emfb_activate(EMFolderView *emfv, BonoboUIComponent *uic, int act)
 		/* ViewThreaded */
 		if (emfv->folder
 		    && (sstate = camel_object_meta_get(emfv->folder, "evolution:thread_list"))) {
-			state = sstate[0] == '1';
+			state = sstate[0] != '0';
 			g_free(sstate);
 		} else {
 			state = gconf_client_get_bool(gconf, "/apps/evolution/mail/display/thread_list", NULL);
