@@ -108,6 +108,15 @@ em_folder_selector_init (EMFolderSelector *emfs)
 static void
 em_folder_selector_destroy (GtkObject *obj)
 {
+	EMFolderSelector *emfs = (EMFolderSelector *) obj;
+	EMFolderTreeModel *model;
+	
+	if (emfs->created_id != 0) {
+		model = em_folder_tree_get_model (emfs->emft);
+		g_signal_handler_disconnect (model, emfs->created_id);
+		emfs->created_id = 0;
+	}
+	
 	GTK_OBJECT_CLASS (parent_class)->destroy (obj);
 }
 
@@ -118,10 +127,30 @@ em_folder_selector_finalize (GObject *obj)
 	
 	g_free (emfs->selected_path);
 	g_free (emfs->selected_uri);
+	g_free (emfs->created_uri);
 	
 	G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
 
+static void
+folder_created_cb (EMFolderTreeModel *model, const char *path, const char *uri, EMFolderSelector *emfs)
+{
+	CamelException ex;
+	CamelStore *store;
+	
+	camel_exception_init (&ex);
+	if (!(store = (CamelStore *) camel_session_get_service (session, uri, CAMEL_PROVIDER_STORE, &ex)))
+		return;
+	
+	if (camel_store_folder_uri_equal (store, emfs->created_uri, uri)) {
+		em_folder_tree_set_selected (emfs->emft, uri);
+		g_signal_handler_disconnect (model, emfs->created_id);
+		emfs->created_id = 0;
+	}
+	
+	camel_object_unref (store);
+}
+
 static void
 emfs_response (GtkWidget *dialog, int response, EMFolderSelector *emfs)
 {
@@ -136,13 +165,20 @@ emfs_response (GtkWidget *dialog, int response, EMFolderSelector *emfs)
 	emft = (EMFolderTree *) em_folder_tree_new_with_model (model);
 	dialog = em_folder_selector_create_new (emft, 0, _("Create New Folder"), _("Specify where to create the folder:"));
 	gtk_window_set_transient_for ((GtkWindow *) dialog, (GtkWindow *) emfs);
+	uri = em_folder_selector_get_selected_uri (emfs);
+	em_folder_tree_set_selected (emft, uri);
 	
 	if (gtk_dialog_run ((GtkDialog *) dialog) == GTK_RESPONSE_OK) {
 		uri = em_folder_selector_get_selected_uri ((EMFolderSelector *) dialog);
 		path = em_folder_selector_get_selected_path ((EMFolderSelector *) dialog);
 		
-		if (em_folder_tree_create_folder (emfs->emft, path, uri))
-			em_folder_tree_set_selected (emfs->emft, uri);
+		g_free (emfs->created_uri);
+		emfs->created_uri = g_strdup (uri);
+		
+		if (emfs->created_id == 0)
+			emfs->created_id = g_signal_connect (model, "folder-added", G_CALLBACK (folder_created_cb), emfs);
+		
+		em_folder_tree_create_folder (emfs->emft, path, uri);
 	}
 	
 	gtk_widget_destroy (dialog);
@@ -175,14 +211,18 @@ folder_selected_cb (EMFolderTree *emft, const char *path, const char *uri, EMFol
 		gtk_dialog_set_response_sensitive (GTK_DIALOG (emfs), GTK_RESPONSE_OK, TRUE);
 }
 
+static void
+folder_activated_cb (EMFolderTree *emft, const char *path, const char *uri, EMFolderSelector *emfs)
+{
+	gtk_dialog_response ((GtkDialog *) emfs, GTK_RESPONSE_OK);
+}
+
 void
 em_folder_selector_construct (EMFolderSelector *emfs, EMFolderTree *emft, guint32 flags, const char *title, const char *text)
 {
-	GtkWidget *scrolled_window;
 	GtkWidget *label;
 	
 	gtk_window_set_default_size (GTK_WINDOW (emfs), 350, 300);
-	gtk_window_set_modal (GTK_WINDOW (emfs), TRUE);
 	gtk_window_set_title (GTK_WINDOW (emfs), title);
 	gtk_container_set_border_width (GTK_CONTAINER (emfs), 6);
 	
@@ -205,16 +245,8 @@ em_folder_selector_construct (EMFolderSelector *emfs, EMFolderTree *emft, guint3
 	gtk_widget_show ((GtkWidget *) emft);
 	
 	g_signal_connect (emfs->emft, "folder-selected", G_CALLBACK (folder_selected_cb), emfs);
-	
-	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
-	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window), GTK_SHADOW_IN);
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
-					GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_widget_show (scrolled_window);
-	
-	gtk_container_add (GTK_CONTAINER (scrolled_window), (GtkWidget *) emft);
-        
-	gtk_box_pack_end (GTK_BOX (GTK_DIALOG (emfs)->vbox), scrolled_window, TRUE, TRUE, 6);
+	g_signal_connect (emfs->emft, "folder-activated", G_CALLBACK (folder_activated_cb), emfs);
+	gtk_box_pack_end (GTK_BOX (GTK_DIALOG (emfs)->vbox), (GtkWidget *)emft, TRUE, TRUE, 6);
 	
 	if (text != NULL) {
 		label = gtk_label_new (text);
@@ -224,7 +256,6 @@ em_folder_selector_construct (EMFolderSelector *emfs, EMFolderTree *emft, guint3
 		gtk_box_pack_end (GTK_BOX (GTK_DIALOG (emfs)->vbox), label, FALSE, TRUE, 6);
 	}
 	
-	GTK_WIDGET_SET_FLAGS ((GtkWidget *) emfs->emft, GTK_CAN_FOCUS);
 	gtk_widget_grab_focus ((GtkWidget *) emfs->emft);
 }
 
@@ -243,7 +274,15 @@ em_folder_selector_new (EMFolderTree *emft, guint32 flags, const char *title, co
 static void
 emfs_create_name_activate (GtkEntry *entry, EMFolderSelector *emfs)
 {
-	g_signal_emit_by_name (emfs, "response", GTK_RESPONSE_OK);
+	if (emfs->name_entry->text_length > 0) {
+		const char *path, *text;
+		
+		text = gtk_entry_get_text (emfs->name_entry);
+		path = em_folder_tree_get_selected_path (emfs->emft);
+		
+		if (text && path && !strchr (text, '/'))
+			g_signal_emit_by_name (emfs, "response", GTK_RESPONSE_OK);
+	}
 }
 
 GtkWidget *
@@ -258,11 +297,13 @@ em_folder_selector_create_new (EMFolderTree *emft, guint32 flags, const char *ti
 	
 	emfs = g_object_new (em_folder_selector_get_type (), NULL);
 	em_folder_selector_construct (emfs, emft, flags, title, text);
+	em_folder_tree_set_excluded(emft, EMFT_EXCLUDE_NOINFERIORS);
 	
 	hbox = gtk_hbox_new (FALSE, 0);
 	w = gtk_label_new_with_mnemonic (_("Folder _name:"));
 	gtk_box_pack_start ((GtkBox *) hbox, w, FALSE, FALSE, 6);
 	emfs->name_entry = (GtkEntry *) gtk_entry_new ();
+	gtk_label_set_mnemonic_widget (GTK_LABEL (w), (GtkWidget *) emfs->name_entry);
 	g_signal_connect (emfs->name_entry, "changed", G_CALLBACK (emfs_create_name_changed), emfs);
 	g_signal_connect (emfs->name_entry, "activate", G_CALLBACK (emfs_create_name_activate), emfs);
 	gtk_box_pack_start ((GtkBox *) hbox, (GtkWidget *) emfs->name_entry, TRUE, FALSE, 6);
@@ -270,6 +311,8 @@ em_folder_selector_create_new (EMFolderTree *emft, guint32 flags, const char *ti
 	
 	gtk_box_pack_start ((GtkBox *) ((GtkDialog *) emfs)->vbox, hbox, FALSE, TRUE, 0);
 	
+	gtk_widget_grab_focus ((GtkWidget *) emfs->name_entry);
+	
 	return (GtkWidget *) emfs;
 }
 
@@ -298,13 +341,10 @@ em_folder_selector_get_selected_uri (EMFolderSelector *emfs)
 	
 	if (uri && emfs->name_entry) {
 		CamelProvider *provider;
-		CamelException ex;
 		CamelURL *url;
 		char *newpath;
 		
-		camel_exception_init (&ex);
-		provider = camel_session_get_provider (session, uri, &ex);
-		camel_exception_clear (&ex);
+		provider = camel_provider_get(uri, NULL);
 		
 		name = gtk_entry_get_text (emfs->name_entry);
 		
@@ -317,8 +357,8 @@ em_folder_selector_get_selected_uri (EMFolderSelector *emfs)
 			
 			camel_url_set_fragment (url, newpath);
 		} else {
-			newpath = g_strdup_printf ("%s/%s", url->path ? url->path : "", name);
-			camel_url_set_path (url, newpath);
+			newpath = g_strdup_printf("%s/%s", (url->path == NULL || strcmp(url->path, "/") == 0) ? "":url->path, name);
+			camel_url_set_path(url, newpath);
 		}
 		
 		g_free (emfs->selected_path);
