@@ -27,8 +27,12 @@
 #include <gtk/gtktreednd.h>
 #include <gtk/gtktreestore.h>
 
+#include <libxml/tree.h>
+
 #include <camel/camel-store.h>
 
+#include <e-util/e-account-list.h>
+
 #ifdef __cplusplus
 extern "C" {
 #pragma }
@@ -49,10 +53,10 @@ enum {
 	COL_STRING_DISPLAY_NAME,  /* string that appears in the tree */
 	COL_POINTER_CAMEL_STORE,  /* CamelStore object */
 	COL_STRING_FOLDER_PATH,   /* if node is a folder, the full path of the folder */
-	COL_STRING_URI,           /* the uri to get the store or
-				   * folder object */
+	COL_STRING_URI,           /* the uri to get the store or folder object */
 	COL_UINT_UNREAD,          /* unread count */
-	
+	COL_UINT_FLAGS,		/* FolderInfo.flags */
+
 	COL_BOOL_IS_STORE,        /* toplevel store node? */
 	COL_BOOL_LOAD_SUBDIRS,    /* %TRUE only if the store/folder
 				   * has subfolders which have not yet
@@ -65,6 +69,7 @@ struct _EMFolderTreeModelStoreInfo {
 	CamelStore *store;
 	GtkTreeRowReference *row;
 	GHashTable *path_hash;  /* maps CamelFolderInfo::path's to GtkTreeRowReferences */
+	EAccount *account;
 	
 	char *display_name;
 	
@@ -78,11 +83,16 @@ struct _EMFolderTreeModelStoreInfo {
 struct _EMFolderTreeModel {
 	GtkTreeStore parent_object;
 	
-	char *filename;          /* state filename */
+	char *filename;            /* state filename */
+	xmlDocPtr expanded;        /* saved expanded state from previous session */
 	
-	GHashTable *store_hash;  /* maps CamelStore's to store-info's */
-	GHashTable *uri_hash;    /* maps URI's to GtkTreeRowReferences */
-	GHashTable *expanded;    /* saved expanded state from previous session */
+	GHashTable *store_hash;    /* maps CamelStore's to store-info's */
+	GHashTable *uri_hash;      /* maps URI's to GtkTreeRowReferences */
+	
+	EAccountList *accounts;
+	GHashTable *account_hash;  /* maps accounts to store-info's */
+	gulong account_changed_id;
+	gulong account_removed_id;
 };
 
 struct _EMFolderTreeModelClass {
@@ -92,6 +102,14 @@ struct _EMFolderTreeModelClass {
 	void     (* loading_row)        (EMFolderTreeModel *model,
 					 GtkTreePath *path,
 					 GtkTreeIter *iter);
+	
+	void     (* loaded_row)        (EMFolderTreeModel *model,
+					GtkTreePath *path,
+					GtkTreeIter *iter);
+	
+	void     (* folder_added)       (EMFolderTreeModel *model,
+					 const char *path,
+					 const char *uri);
 };
 
 
@@ -114,6 +132,11 @@ gboolean em_folder_tree_model_get_expanded (EMFolderTreeModel *model, const char
 void em_folder_tree_model_set_expanded (EMFolderTreeModel *model, const char *key, gboolean expanded);
 void em_folder_tree_model_save_expanded (EMFolderTreeModel *model);
 
+typedef void (* EMFTModelExpandFunc) (EMFolderTreeModel *model, const char *path, void *user_data);
+void em_folder_tree_model_expand_foreach (EMFolderTreeModel *model, EMFTModelExpandFunc func, void *user_data);
+
+void em_folder_tree_model_set_unread_count (EMFolderTreeModel *model, CamelStore *store, const char *path, int unread);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
