@@ -58,7 +58,9 @@ static void emf_builtin_init(EMFormatClass *);
 static const char *emf_snoop_part(CamelMimePart *part);
 
 static const EMFormatHandler *emf_find_handler(EMFormat *emf, const char *mime_type);
-static void emf_format_clone(EMFormat *emf, CamelMedium *msg, EMFormat *emfsource);
+static void emf_format_clone(EMFormat *emf, CamelFolder *folder, const char *uid, CamelMimeMessage *msg, EMFormat *emfsource);
+static void emf_format_prefix(EMFormat *emf, CamelStream *stream);
+static void emf_format_secure(EMFormat *emf, CamelStream *stream, CamelMimePart *part, CamelCipherValidity *valid);
 static gboolean emf_busy(EMFormat *emf);
 
 enum {
@@ -77,6 +79,7 @@ emf_init(GObject *o)
 	emf->inline_table = g_hash_table_new(NULL, NULL);
 	e_dlist_init(&emf->header_list);
 	em_format_default_headers(emf);
+	emf->part_id = g_string_new("");
 }
 
 static void
@@ -91,8 +94,10 @@ emf_finalise(GObject *o)
 		g_hash_table_destroy(emf->inline_table);
 
 	em_format_clear_headers(emf);
+	camel_cipher_validity_free(emf->valid);
 	g_free(emf->charset);
-	
+	g_string_free(emf->part_id, TRUE);
+
 	/* FIXME: check pending jobs */
 	
 	((GObjectClass *)emf_parent)->finalize(o);
@@ -114,6 +119,8 @@ emf_class_init(GObjectClass *klass)
 	klass->finalize = emf_finalise;
 	((EMFormatClass *)klass)->find_handler = emf_find_handler;
 	((EMFormatClass *)klass)->format_clone = emf_format_clone;
+	((EMFormatClass *)klass)->format_prefix = emf_format_prefix;
+	((EMFormatClass *)klass)->format_secure = emf_format_secure;
 	((EMFormatClass *)klass)->busy = emf_busy;
 
 	emf_signals[EMF_COMPLETE] =
@@ -262,7 +269,6 @@ em_format_add_puri(EMFormat *emf, size_t size, const char *cid, CamelMimePart *p
 {
 	EMFormatPURI *puri;
 	const char *tmp;
-	static unsigned int uriid;
 
 	g_assert(size >= sizeof(*puri));
 	puri = g_malloc0(size);
@@ -271,6 +277,7 @@ em_format_add_puri(EMFormat *emf, size_t size, const char *cid, CamelMimePart *p
 	puri->func = func;
 	puri->use_count = 0;
 	puri->cid = g_strdup(cid);
+	puri->part_id = g_strdup(emf->part_id->str);
 
 	if (part) {
 		camel_object_ref(part);
@@ -282,7 +289,7 @@ em_format_add_puri(EMFormat *emf, size_t size, const char *cid, CamelMimePart *p
 		if (tmp)
 			puri->cid = g_strdup_printf("cid:%s", tmp);
 		else
-			puri->cid = g_strdup_printf("em-no-cid-%u", uriid++);
+			puri->cid = g_strdup_printf("em-no-cid:%s", emf->part_id->str);
 
 		d(printf("built cid '%s'\n", puri->cid));
 
@@ -421,6 +428,7 @@ emf_clear_puri_node(struct _EMFormatPURITree *node)
 		while (pn) {
 			g_free(pw->uri);
 			g_free(pw->cid);
+			g_free(pw->part_id);
 			if (pw->part)
 				camel_object_unref(pw->part);
 			g_free(pw);
@@ -472,10 +480,13 @@ void
 em_format_part_as(EMFormat *emf, CamelStream *stream, CamelMimePart *part, const char *mime_type)
 {
 	const EMFormatHandler *handle = NULL;
+	const char *snoop_save = emf->snoop_mime_type;
+
+	emf->snoop_mime_type = NULL;
 
 	if (mime_type != NULL) {
 		if (g_ascii_strcasecmp(mime_type, "application/octet-stream") == 0)
-			mime_type = emf_snoop_part(part);
+			emf->snoop_mime_type = mime_type = emf_snoop_part(part);
 
 		handle = em_format_find_handler(emf, mime_type);
 		if (handle == NULL)
@@ -485,6 +496,7 @@ em_format_part_as(EMFormat *emf, CamelStream *stream, CamelMimePart *part, const
 		    && !em_format_is_attachment(emf, part)) {
 			d(printf("running handler for type '%s'\n", mime_type));
 			handle->handler(emf, stream, part, handle);
+			emf->snoop_mime_type = snoop_save;
 			return;
 		}
 		d(printf("this type is an attachment? '%s'\n", mime_type));
@@ -493,6 +505,7 @@ em_format_part_as(EMFormat *emf, CamelStream *stream, CamelMimePart *part, const
 	}
 
 	((EMFormatClass *)G_OBJECT_GET_CLASS(emf))->format_attachment(emf, stream, part, mime_type, handle);
+	emf->snoop_mime_type = snoop_save;
 }
 
 void
@@ -515,7 +528,7 @@ emf_clone_inlines(void *key, void *val, void *data)
 }
 
 static void
-emf_format_clone(EMFormat *emf, CamelMedium *msg, EMFormat *emfsource)
+emf_format_clone(EMFormat *emf, CamelFolder *folder, const char *uid, CamelMimeMessage *msg, EMFormat *emfsource)
 {
 	em_format_clear_puri_tree(emf);
 
@@ -532,6 +545,20 @@ emf_format_clone(EMFormat *emf, CamelMedium *msg, EMFormat *emfsource)
 		}
 	}
 
+	/* what a mess */
+	if (folder != emf->folder) {
+		if (emf->folder)
+			camel_object_unref(emf->folder);
+		if (folder)
+			camel_object_ref(folder);
+		emf->folder = folder;
+	}
+
+	if (uid != emf->uid) {
+		g_free(emf->uid);
+		emf->uid = g_strdup(uid);
+	}
+
 	if (msg != emf->message) {
 		if (emf->message)
 			camel_object_unref(emf->message);
@@ -539,6 +566,46 @@ emf_format_clone(EMFormat *emf, CamelMedium *msg, EMFormat *emfsource)
 			camel_object_ref(msg);
 		emf->message = msg;
 	}
+
+	g_string_truncate(emf->part_id, 0);
+	if (folder != NULL)
+		/* TODO build some string based on the folder name/location? */
+		g_string_append_printf(emf->part_id, ".%p", folder);
+	if (uid != NULL)
+		g_string_append_printf(emf->part_id, ".%s", uid);
+}
+
+static void
+emf_format_prefix(EMFormat *emf, CamelStream *stream)
+{
+	/* NOOP */
+}
+
+static void
+emf_format_secure(EMFormat *emf, CamelStream *stream, CamelMimePart *part, CamelCipherValidity *valid)
+{
+	CamelCipherValidity *save = emf->valid_parent;
+	int len;
+
+	/* Note that this also requires support from higher up in the class chain
+	    - validity needs to be cleared when you start output
+	    - also needs to be cleared (but saved) whenever you start a new message. */
+
+	if (emf->valid == NULL) {
+		emf->valid = valid;
+	} else {
+		e_dlist_addtail(&emf->valid_parent->children, (EDListNode *)valid);
+		camel_cipher_validity_envelope(emf->valid_parent, valid);
+	}
+
+	emf->valid_parent = valid;
+
+	len = emf->part_id->len;
+	g_string_append_printf(emf->part_id, ".secured");
+	em_format_part(emf, stream, part);
+	g_string_truncate(emf->part_id, len);
+
+	emf->valid_parent = save;
 }
 
 static gboolean
@@ -550,7 +617,9 @@ emf_busy(EMFormat *emf)
 /**
  * em_format_format_clone:
  * @emf: Mail formatter.
- * @msg: Mail message.
+ * @folder: Camel Folder.
+ * @uid: Uid of message.
+ * @msg: Camel Message.
  * @emfsource: Used as a basis for user-altered layout, e.g. inline viewed
  * attachments.
  * 
@@ -600,7 +669,7 @@ em_format_set_mode(EMFormat *emf, em_format_mode_t type)
 
 	/* force redraw if type changed afterwards */
 	if (emf->message)
-		em_format_format_clone(emf, emf->message, emf);
+		em_format_redraw(emf);
 }
 
 /**
@@ -623,10 +692,9 @@ em_format_set_charset(EMFormat *emf, const char *charset)
 	emf->charset = g_strdup(charset);
 
 	if (emf->message)
-		em_format_format_clone(emf, emf->message, emf);
+		em_format_redraw(emf);
 }
 
-
 /**
  * em_format_set_default_charset:
  * @emf: 
@@ -648,7 +716,7 @@ em_format_set_default_charset(EMFormat *emf, const char *charset)
 	emf->default_charset = g_strdup(charset);
 
 	if (emf->message && emf->charset == NULL)
-		em_format_format_clone(emf, emf->message, emf);
+		em_format_redraw(emf);
 }
 
 /**
@@ -667,6 +735,7 @@ em_format_clear_headers(EMFormat *emf)
 		g_free(eh);
 }
 
+/* note: also copied in em-mailer-prefs.c */
 static const struct {
 	const char *name;
 	guint32 flags;
@@ -678,6 +747,7 @@ static const struct {
 	{ N_("Bcc"), EM_FORMAT_HEADER_BOLD },
 	{ N_("Subject"), EM_FORMAT_HEADER_BOLD },
 	{ N_("Date"), EM_FORMAT_HEADER_BOLD },
+	{ N_("Newsgroups"), EM_FORMAT_HEADER_BOLD },
 	{ "x-evolution-mailer", 0 }, /* DO NOT translate */
 };
 
@@ -753,39 +823,38 @@ int em_format_is_attachment(EMFormat *emf, CamelMimePart *part)
  * em_format_is_inline:
  * @emf: 
  * @part: 
+ * @handle: handler for this part
  * 
  * Returns true if the part should be displayed inline.  Any part with
- * a Content-Disposition of inline, or any message type is displayed
- * inline.
+ * a Content-Disposition of inline, or if the @handle has a default
+ * inline set, will be shown inline.
  *
- * ::set_inline() called on the same part will override any calculated
+ * :set_inline() called on the same part will override any calculated
  * value.
  * 
  * Return value: 
  **/
-int em_format_is_inline(EMFormat *emf, CamelMimePart *part)
+int em_format_is_inline(EMFormat *emf, CamelMimePart *part, const EMFormatHandler *handle)
 {
 	void *dummy, *override;
 	const char *tmp;
-	CamelContentType *ct;
+
+	if (handle == NULL)
+		return FALSE;
 
 	if (g_hash_table_lookup_extended(emf->inline_table, part, &dummy, &override))
 		return GPOINTER_TO_INT(override);
 
-	ct = camel_mime_part_get_content_type(part);
-
-	/* TODO: make this depend on libnss supported */
-	/* For some reason rfc2633 says we always add this as an attachment, which
-	   stuffs us up since we don't want to treat it that way at all ... */
-	if (camel_content_type_is(ct, "application", "x-pkcs7-mime"))
+	/* some types need to override the disposition, e.g. application/x-pkcs7-mime */
+	if (handle->flags & EM_FORMAT_HANDLER_INLINE_DISPOSITION)
 		return TRUE;
 
 	tmp = camel_mime_part_get_disposition(part);
 	if (tmp)
 		return g_ascii_strcasecmp(tmp, "inline") == 0;
 
-	/* messages are always inline? */
-	return camel_content_type_is (ct, "message", "*");
+	/* otherwise, use the default for this handler type */
+	return (handle->flags & EM_FORMAT_HANDLER_INLINE) != 0;
 }
 
 /**
@@ -803,6 +872,28 @@ void em_format_set_inline(EMFormat *emf, CamelMimePart *part, int state)
 	g_hash_table_insert(emf->inline_table, part, GINT_TO_POINTER(state));
 }
 
+void em_format_format_error(EMFormat *emf, CamelStream *stream, const char *fmt, ...)
+{
+	va_list ap;
+	char *txt;
+
+	va_start(ap, fmt);
+	txt = g_strdup_vprintf(fmt, ap);
+	((EMFormatClass *)G_OBJECT_GET_CLASS(emf))->format_error((emf), (stream), (txt));
+	g_free(txt);
+}
+
+void
+em_format_format_secure(EMFormat *emf, struct _CamelStream *stream, struct _CamelMimePart *part, struct _CamelCipherValidity *valid)
+{
+	((EMFormatClass *)G_OBJECT_GET_CLASS(emf))->format_secure(emf, stream, part, valid);
+
+	if (emf->valid_parent == NULL && emf->valid != NULL) {
+		camel_cipher_validity_free(emf->valid);
+		emf->valid = NULL;
+	}
+}
+
 /* should this be virtual? */
 void
 em_format_format_content(EMFormat *emf, CamelStream *stream, CamelMimePart *part)
@@ -955,19 +1046,53 @@ emf_snoop_part(CamelMimePart *part)
 	   see bug #11778 for some discussion */
 }
 
+#ifdef ENABLE_SMIME
+static void
+emf_application_xpkcs7mime(EMFormat *emf, CamelStream *stream, CamelMimePart *part, const EMFormatHandler *info)
+{
+	CamelCipherContext *context;
+	CamelException *ex;
+	extern CamelSession *session;
+	CamelMimePart *opart;
+	CamelCipherValidity *valid;
+
+	ex = camel_exception_new();
+
+	context = camel_smime_context_new(session);
+
+	opart = camel_mime_part_new();
+	valid = camel_cipher_decrypt(context, part, opart, ex);
+	if (valid == NULL) {
+		em_format_format_error(emf, stream, ex->desc?ex->desc:_("Could not parse S/MIME message: Unknown error"));
+		em_format_part_as(emf, stream, part, NULL);
+	} else {
+		em_format_format_secure(emf, stream, opart, valid);
+	}
+
+	camel_object_unref(opart);
+	camel_object_unref(context);
+	camel_exception_free(ex);
+}
+#endif
+
 /* RFC 1740 */
 static void
 emf_multipart_appledouble(EMFormat *emf, CamelStream *stream, CamelMimePart *part, const EMFormatHandler *info)
 {
 	CamelMultipart *mp = (CamelMultipart *)camel_medium_get_content_object((CamelMedium *)part);
-	
+	int len;
+
 	if (!CAMEL_IS_MULTIPART(mp)) {
+		em_format_format_error(emf, stream, _("Internal error: expecting CamelMultipart, got \"%s\""), ((CamelObject *)mp)->klass->name);
 		em_format_format_source(emf, stream, part);
 		return;
 	}
 
 	/* try the data fork for something useful, doubtful but who knows */
+	len = emf->part_id->len;
+	g_string_append_printf(emf->part_id, ".appledouble.1");
 	em_format_part(emf, stream, camel_multipart_get_part(mp, 1));
+	g_string_truncate(emf->part_id, len);
 }
 
 /* RFC ??? */
@@ -975,17 +1100,21 @@ static void
 emf_multipart_mixed(EMFormat *emf, CamelStream *stream, CamelMimePart *part, const EMFormatHandler *info)
 {
 	CamelMultipart *mp = (CamelMultipart *)camel_medium_get_content_object((CamelMedium *)part);
-	int i, nparts;
+	int i, nparts, len;
 
 	if (!CAMEL_IS_MULTIPART(mp)) {
+		em_format_format_error(emf, stream, _("Internal error: expecting CamelMultipart, got \"%s\""), ((CamelObject *)mp)->klass->name);
 		em_format_format_source(emf, stream, part);
 		return;
 	}
-	
+
+	len = emf->part_id->len;
 	nparts = camel_multipart_get_number(mp);	
 	for (i = 0; i < nparts; i++) {
 		part = camel_multipart_get_part(mp, i);
+		g_string_append_printf(emf->part_id, ".mixed.%d", i);
 		em_format_part(emf, stream, part);
+		g_string_truncate(emf->part_id, len);
 	}
 }
 
@@ -994,10 +1123,11 @@ static void
 emf_multipart_alternative(EMFormat *emf, CamelStream *stream, CamelMimePart *part, const EMFormatHandler *info)
 {
 	CamelMultipart *mp = (CamelMultipart *)camel_medium_get_content_object((CamelMedium *)part);
-	int i, nparts;
+	int i, nparts, bestid;
 	CamelMimePart *best = NULL;
 
 	if (!CAMEL_IS_MULTIPART(mp)) {
+		em_format_format_error(emf, stream, _("Internal error: expecting CamelMultipart, got \"%s\""), ((CamelObject *)mp)->klass->name);
 		em_format_format_source(emf, stream, part);
 		return;
 	}
@@ -1015,53 +1145,57 @@ emf_multipart_alternative(EMFormat *emf, CamelStream *stream, CamelMimePart *par
 		  return part;*/
 
 		if (em_format_find_handler(emf, mime_type)
-		    || (best == NULL && em_format_fallback_handler(emf, mime_type)))
+		    || (best == NULL && em_format_fallback_handler(emf, mime_type))) {
 			best = part;
+			bestid = i;
+		}
 
 		g_free(mime_type);
 	}
 
-	if (best)
+	if (best) {
+		int len = emf->part_id->len;
+
+		g_string_append_printf(emf->part_id, ".alternative.%d", bestid);
 		em_format_part(emf, stream, best);
-	else
+		g_string_truncate(emf->part_id, len);
+	} else
 		emf_multipart_mixed(emf, stream, part, info);
 }
 
 static void
 emf_multipart_encrypted(EMFormat *emf, CamelStream *stream, CamelMimePart *part, const EMFormatHandler *info)
 {
-	CamelMultipartEncrypted *mpe;
-	CamelMimePart *mime_part;
-	CamelCipherContext *cipher;
-	CamelException ex;
+	CamelCipherContext *context;
+	CamelException *ex;
 	const char *protocol;
+	CamelMimePart *opart;
+	CamelCipherValidity *valid;
 
 	/* Currently we only handle RFC2015-style PGP encryption. */
 	protocol = camel_content_type_param (((CamelDataWrapper *) part)->mime_type, "protocol");
-	if (!protocol || strcmp (protocol, "application/pgp-encrypted") != 0)
-		return emf_multipart_mixed(emf, stream, part, info);
-	
-	mpe = (CamelMultipartEncrypted *)camel_medium_get_content_object((CamelMedium *)part);
-
-	if (!CAMEL_IS_MULTIPART_ENCRYPTED(mpe)) {
-		em_format_format_source(emf, stream, part);
+	if (!protocol || g_ascii_strcasecmp (protocol, "application/pgp-encrypted") != 0) {
+		em_format_format_error(emf, stream, _("Unsupported encryption type for multipart/encrypted"));
+		em_format_part_as(emf, stream, part, "multipart/mixed");
 		return;
 	}
 
-	camel_exception_init (&ex);
-	cipher = camel_gpg_context_new(emf->session);
-	mime_part = camel_multipart_encrypted_decrypt(mpe, cipher, &ex);
-	camel_object_unref(cipher);
-	
-	if (camel_exception_is_set(&ex)) {
-		/* FIXME: error handler */
-		em_format_format_error(emf, stream, camel_exception_get_description(&ex));
-		camel_exception_clear(&ex);
-		return;
+	ex = camel_exception_new();
+	context = camel_gpg_context_new(emf->session);
+	opart = camel_mime_part_new();
+	valid = camel_cipher_decrypt(context, part, opart, ex);
+	if (valid == NULL) {
+		em_format_format_error(emf, stream, ex->desc?("Could not parse S/MIME message"):_("Could not parse S/MIME message: Unknown error"));
+		if (ex->desc)
+			em_format_format_error(emf, stream, ex->desc);
+		em_format_part_as(emf, stream, part, "multipart/mixed");
+	} else {
+		em_format_format_secure(emf, stream, opart, valid);
 	}
 
-	em_format_part(emf, stream, mime_part);
-	camel_object_unref(mime_part);
+	camel_object_unref(opart);
+	camel_object_unref(context);
+	camel_exception_free(ex);
 }
 
 static void
@@ -1079,12 +1213,14 @@ emf_multipart_related(EMFormat *emf, CamelStream *stream, CamelMimePart *part, c
 	CamelMimePart *body_part, *display_part = NULL;
 	CamelContentType *content_type;
 	const char *location, *start;
-	int i, nparts;
+	int i, nparts, partidlen, displayid = 0;
+	char *oldpartid;
 	CamelURL *base_save = NULL;
 	struct _EMFormatPURITree *ptree;
 	EMFormatPURI *puri, *purin;
 
 	if (!CAMEL_IS_MULTIPART(mp)) {
+		em_format_format_error(emf, stream, _("Internal error: expecting CamelMultipart, got \"%s\""), ((CamelObject *)mp)->klass->name);
 		em_format_format_source(emf, stream, part);
 		return;
 	}
@@ -1107,6 +1243,7 @@ emf_multipart_related(EMFormat *emf, CamelStream *stream, CamelMimePart *part, c
 			
 			if (cid && !strncmp(cid, start, len) && strlen(cid) == len) {
 				display_part = body_part;
+				displayid = i;
 				break;
 			}
 		}
@@ -1128,16 +1265,24 @@ emf_multipart_related(EMFormat *emf, CamelStream *stream, CamelMimePart *part, c
 	}
 	em_format_push_level(emf);
 
+	oldpartid = g_strdup(emf->part_id->str);
+	partidlen = emf->part_id->len;
+
 	/* queue up the parts for possible inclusion */
 	for (i = 0; i < nparts; i++) {
 		body_part = camel_multipart_get_part(mp, i);
 		if (body_part != display_part) {
+			/* set the partid since add_puri uses it */
+			g_string_append_printf(emf->part_id, ".related.%d", i);
 			puri = em_format_add_puri(emf, sizeof(EMFormatPURI), NULL, body_part, emf_write_related);
+			g_string_truncate(emf->part_id, partidlen);
 			d(printf(" part '%s' '%s' added\n", puri->uri?puri->uri:"", puri->cid));
 		}
 	}
 	
+	g_string_append_printf(emf->part_id, ".related.%d", displayid);
 	em_format_part(emf, stream, display_part);
+	g_string_truncate(emf->part_id, partidlen);
 	camel_stream_flush(stream);
 
 	ptree = emf->pending_uri_level;
@@ -1146,14 +1291,19 @@ emf_multipart_related(EMFormat *emf, CamelStream *stream, CamelMimePart *part, c
 	while (purin) {
 		if (purin->use_count == 0) {
 			d(printf("part '%s' '%s' used '%d'\n", purin->uri?purin->uri:"", purin->cid, purin->use_count));
-			if (purin->func == emf_write_related)
+			if (purin->func == emf_write_related) {
+				g_string_printf(emf->part_id, "%s", puri->part_id);
 				em_format_part(emf, stream, puri->part);
-			else
+			} else
 				printf("unreferenced uri generated by format code: %s\n", purin->uri?purin->uri:purin->cid);
 		}
 		puri = purin;
 		purin = purin->next;
 	}
+
+	g_string_printf(emf->part_id, "%s", oldpartid);
+	g_free(oldpartid);
+
 	em_format_pull_level(emf);
 	
 	if (location) {
@@ -1162,82 +1312,84 @@ emf_multipart_related(EMFormat *emf, CamelStream *stream, CamelMimePart *part, c
 	}
 }
 
-/* this is only a fallback implementation, implementations should override */
 static void
 emf_multipart_signed(EMFormat *emf, CamelStream *stream, CamelMimePart *part, const EMFormatHandler *info)
 {
 	CamelMimePart *cpart;
 	CamelMultipartSigned *mps;
-	CamelCipherValidity *valid = NULL;
-	CamelException ex;
-	const char *message = NULL;
-	gboolean good = FALSE;
+	CamelCipherContext *cipher = NULL;
 
 	mps = (CamelMultipartSigned *)camel_medium_get_content_object((CamelMedium *)part);
 	if (!CAMEL_IS_MULTIPART_SIGNED(mps)
 	    || (cpart = camel_multipart_get_part((CamelMultipart *)mps, CAMEL_MULTIPART_SIGNED_CONTENT)) == NULL) {
+		em_format_format_error(emf, stream, _("Could not parse MIME message. Displaying as source."));
 		em_format_format_source(emf, stream, part);
 		return;
 	}
 
-	em_format_part(emf, stream, cpart);
-
-	/* FIXME: This sequence is also copied in em-format-html.c */
-
-	camel_exception_init(&ex);
-	if (emf->session == NULL) {
-		message = _("Session not initialised");
-	} else {
-		CamelCipherContext *cipher = NULL;
-
-		/* FIXME: Should be done via a plugin interface */
+	/* FIXME: Should be done via a plugin interface */
+	/* FIXME: duplicated in em-format-html-display.c */
+	if (mps->protocol) {
+#ifdef ENABLE_SMIME
 		if (g_ascii_strcasecmp("application/x-pkcs7-signature", mps->protocol) == 0
 		    || g_ascii_strcasecmp("application/pkcs7-signature", mps->protocol) == 0)
 			cipher = camel_smime_context_new(emf->session);
-		else if (g_ascii_strcasecmp("application/pgp-signature", mps->protocol) == 0)
-			cipher = camel_gpg_context_new(emf->session);
+		else
+#endif
+			if (g_ascii_strcasecmp("application/pgp-signature", mps->protocol) == 0)
+				cipher = camel_gpg_context_new(emf->session);
+	}
 
-		if (cipher == NULL) {
-			message = _("Unsupported signature format");
+	if (cipher == NULL) {
+		em_format_format_error(emf, stream, _("Unsupported signature format"));
+		em_format_part_as(emf, stream, part, "multipart/mixed");
+	} else {
+		CamelException *ex = camel_exception_new();
+		CamelCipherValidity *valid;
+
+		valid = camel_cipher_verify(cipher, part, ex);
+		if (valid == NULL) {
+			em_format_format_error(emf, stream, ex->desc?_("Error verifying signature"):_("Unknown error verifying signature"));
+			if (ex->desc)
+				em_format_format_error(emf, stream, ex->desc);
+			em_format_part_as(emf, stream, part, "multipart/mixed");
 		} else {
-			valid = camel_cipher_verify(cipher, part, &ex);
-			camel_object_unref(cipher);
-			if (valid) {
-				good = camel_cipher_validity_get_valid(valid);
-				message = camel_cipher_validity_get_description(valid);
-			} else {
-				message = camel_exception_get_description(&ex);
-			}
+			em_format_format_secure(emf, stream, cpart, valid);
 		}
+
+		camel_exception_free(ex);
+		camel_object_unref(cipher);
 	}
-	
-	if (good)
-		em_format_format_error(emf, stream, _("This message is digitally signed and has been found to be authentic."));
-	else
-		em_format_format_error(emf, stream, _("This message is digitally signed but can not be proven to be authentic."));
-	
-	if (message)
-		em_format_format_error(emf, stream, message);
-	
-	camel_exception_clear(&ex);
-	camel_cipher_validity_free(valid);
 }
 
-/* this is only a fallback, any implementer should implement */
 static void
 emf_message_rfc822(EMFormat *emf, CamelStream *stream, CamelMimePart *part, const EMFormatHandler *info)
 {
 	CamelDataWrapper *dw = camel_medium_get_content_object((CamelMedium *)part);
+	int len;
 
 	if (!CAMEL_IS_MIME_MESSAGE(dw)) {
+		em_format_format_error(emf, stream, _("Internal error: expecting CamelMimeMessage, got \"%s\""), ((CamelObject *)dw)->klass->name);
 		em_format_format_source(emf, stream, part);
 		return;
 	}
 
+	len = emf->part_id->len;
+	g_string_append_printf(emf->part_id, ".rfc822");
 	em_format_format_message(emf, stream, (CamelMedium *)dw);
+	g_string_truncate(emf->part_id, len);
+}
+
+static void
+emf_message_deliverystatus(EMFormat *emf, CamelStream *stream, CamelMimePart *part, const EMFormatHandler *info)
+{
+	em_format_format_text(emf, stream, camel_medium_get_content_object((CamelMedium *)part));
 }
 
 static EMFormatHandler type_builtin_table[] = {
+#ifdef ENABLE_SMIME
+	{ "application/x-pkcs7-mime", (EMFormatFunc)emf_application_xpkcs7mime, EM_FORMAT_HANDLER_INLINE_DISPOSITION },
+#endif
 	{ "multipart/alternative", emf_multipart_alternative },
 	{ "multipart/appledouble", emf_multipart_appledouble },
 	{ "multipart/encrypted", emf_multipart_encrypted },
@@ -1245,9 +1397,16 @@ static EMFormatHandler type_builtin_table[] = {
 	{ "multipart/signed", emf_multipart_signed },
 	{ "multipart/related", emf_multipart_related },
 	{ "multipart/*", emf_multipart_mixed },
-	{ "message/rfc822", emf_message_rfc822 },
-	{ "message/news", emf_message_rfc822 },
-	{ "message/*", emf_message_rfc822 },
+	{ "message/rfc822", emf_message_rfc822, EM_FORMAT_HANDLER_INLINE },
+	{ "message/news", emf_message_rfc822, EM_FORMAT_HANDLER_INLINE },
+	{ "message/delivery-status", emf_message_deliverystatus },
+	{ "message/*", emf_message_rfc822, EM_FORMAT_HANDLER_INLINE },
+
+	/* Insert brokenly-named parts here */
+#ifdef ENABLE_SMIME
+	{ "application/pkcs7-mime", (EMFormatFunc)emf_application_xpkcs7mime, EM_FORMAT_HANDLER_INLINE_DISPOSITION },
+#endif
+
 };
 
 static void
