@@ -32,9 +32,15 @@
 #include <gtk/gtkmain.h>
 #include <gtk/gtkdialog.h>
 #include <gtk/gtkstock.h>
+#include <gtk/gtkprogressbar.h>
+#include <gtk/gtktable.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtkimage.h>
+#include <gtk/gtkbox.h>
 #include <libgnomeui/gnome-window-icon.h>
 
 #include "e-util/e-gtk-utils.h"
+#include "e-util/e-account-list.h"
 
 #include "widgets/misc/e-clipped-label.h"
 #include "filter/filter-filter.h"
@@ -42,7 +48,6 @@
 #include "camel/camel-folder.h"
 #include "camel/camel-operation.h"
 
-#include "mail.h"
 #include "mail-mt.h"
 #include "mail-component.h"
 #include "mail-config.h"
@@ -112,6 +117,8 @@ struct _send_info {
 	GtkButton *stop;
 	EClippedLabel *status;
 
+	int again;		/* need to run send again */
+
 	int timeout_id;
 	char *what;
 	int pc;
@@ -120,6 +127,8 @@ struct _send_info {
 	struct _send_data *data;
 };
 
+static CamelFolder *receive_get_folder(CamelFilterDriver *d, const char *uri, void *data, CamelException *ex);
+
 static struct _send_data *send_data = NULL;
 static GtkWidget *send_recv_dialog = NULL;
 
@@ -131,7 +140,8 @@ static struct _send_data *setup_send_data(void)
 		send_data = data = g_malloc0(sizeof(*data));
 		data->lock = g_mutex_new();
 		data->folders = g_hash_table_new(g_str_hash, g_str_equal);
-		data->inbox = mail_tool_get_local_inbox(NULL);
+		data->inbox = mail_component_get_folder(NULL, MAIL_COMPONENT_FOLDER_LOCAL_INBOX);
+		camel_object_ref(data->inbox);
 		data->active = g_hash_table_new(g_str_hash, g_str_equal);
 	}
 	return send_data;
@@ -264,11 +274,13 @@ format_url(const char *internal_url)
 static send_info_t get_receive_type(const char *url)
 {
 	CamelProvider *provider;
-	CamelException ex;
 
-	camel_exception_init(&ex);
-	provider = camel_session_get_provider (session, url, &ex);
-	camel_exception_clear(&ex);
+	/* HACK: since mbox is ALSO used for native evolution trees now, we need to
+	   fudge this to treat it as a special 'movemail' source */
+	if (!strncmp(url, "mbox:", 5))
+		return SEND_RECEIVE;
+
+	provider = camel_provider_get(url, NULL);
 	if (!provider)
 		return SEND_INVALID;
 	
@@ -297,7 +309,6 @@ build_dialog (EAccountList *accounts, CamelFolder *outbox, const char *destinati
 	EClippedLabel *status_label;
 	GtkProgressBar *bar;
 	GtkButton *stop;
-	GtkHSeparator *line;
 	struct _send_info *info;
 	char *pretty_url;
 	EAccount *account;
@@ -566,6 +577,19 @@ receive_done (char *uri, void *data)
 {
 	struct _send_info *info = data;
 
+	/* if we've been called to run again - run again */
+	if (info->type == SEND_SEND && info->state == SEND_ACTIVE && info->again) {
+		info->again = 0;
+		mail_send_queue (mail_component_get_folder(NULL, MAIL_COMPONENT_FOLDER_OUTBOX),
+				 info->uri,
+				 FILTER_SOURCE_OUTGOING,
+				 info->cancel,
+				 receive_get_folder, info,
+				 receive_status, info,
+				 receive_done, info);
+		return;
+	}
+
 	if (info->bar) {
 		gtk_progress_bar_set_fraction((GtkProgressBar *)info->bar, (gfloat)1.0);
 
@@ -671,7 +695,7 @@ receive_update_got_store (char *uri, CamelStore *store, void *data)
 
 GtkWidget *mail_send_receive (void)
 {
-	extern CamelFolder *outbox_folder;
+	CamelFolder *outbox_folder;
 	struct _send_data *data;
 	EAccountList *accounts;
 	EAccount *account;
@@ -693,7 +717,8 @@ GtkWidget *mail_send_receive (void)
 		return send_recv_dialog;
 	
 	accounts = mail_config_get_accounts ();
-	
+
+	outbox_folder = mail_component_get_folder(NULL, MAIL_COMPONENT_FOLDER_OUTBOX);
 	data = build_dialog (accounts, outbox_folder, account->transport->url);
 	scan = data->infos;
 	while (scan) {
@@ -771,7 +796,7 @@ mail_autoreceive_setup (void)
 	EAccountList *accounts;
 	GHashTable *set_hash;
 	EIterator *iter;
-	
+
 	accounts = mail_config_get_accounts ();
 	
 	if (auto_active == NULL)
@@ -791,7 +816,7 @@ mail_autoreceive_setup (void)
 		if (account->enabled && source->url && source->auto_check) {
 			struct _auto_data *info;
 			
-			d(printf("setting up auto-receive mail for : %s\n", source->url));
+			d(printf("setting up auto-receive mail for : %s %dm\n", source->url, source->auto_check_time));
 			
 			g_hash_table_remove(set_hash, source->url);
 			info = g_hash_table_lookup(auto_active, source->url);
@@ -831,7 +856,7 @@ mail_receive_uri (const char *uri, int keep)
 {
 	struct _send_info *info;
 	struct _send_data *data;
-	extern CamelFolder *outbox_folder;
+	CamelFolder *outbox_folder;
 	send_info_t type;
 	
 	data = setup_send_data();
@@ -876,6 +901,7 @@ mail_receive_uri (const char *uri, int keep)
 		break;
 	case SEND_SEND:
 		/* todo, store the folder in info? */
+		outbox_folder = mail_component_get_folder(NULL, MAIL_COMPONENT_FOLDER_OUTBOX);
 		mail_send_queue (outbox_folder, info->uri,
 				 FILTER_SOURCE_OUTGOING,
 				 info->cancel,
@@ -894,7 +920,7 @@ mail_receive_uri (const char *uri, int keep)
 void
 mail_send (void)
 {
-	extern CamelFolder *outbox_folder;
+	CamelFolder *outbox_folder;
 	EAccountService *transport;
 	struct _send_info *info;
 	struct _send_data *data;
@@ -907,6 +933,7 @@ mail_send (void)
 	data = setup_send_data ();
 	info = g_hash_table_lookup (data->active, SEND_URI_KEY);
 	if (info != NULL) {
+		info->again++;
 		d(printf("send of %s still in progress\n", transport->url));
 		return;
 	}
@@ -936,6 +963,7 @@ mail_send (void)
 	g_hash_table_insert (data->active, SEND_URI_KEY, info);
 	
 	/* todo, store the folder in info? */
+	outbox_folder = mail_component_get_folder(NULL, MAIL_COMPONENT_FOLDER_OUTBOX);
 	mail_send_queue (outbox_folder, info->uri,
 			 FILTER_SOURCE_OUTGOING,
 			 info->cancel,
