@@ -37,6 +37,7 @@
 
 #include <camel/camel.h>
 #include <camel/camel-vee-folder.h>
+#include <camel/camel-file-utils.h>
 
 #include <filter/vfolder-rule.h>
 #include <filter/vfolder-context.h>
@@ -45,23 +46,17 @@
 
 #include "e-util/e-meta.h"
 
-#include "mail.h" /*session*/
 #include "mail-component.h"
+#include "mail-session.h"
 #include "mail-config.h"
 #include "mail-vfolder.h"
 #include "mail-tools.h"
 #include "mail-mt.h"
 #include "mail-folder-cache.h"
-
+#include "em-utils.h"
 
 /* **************************************** */
 
-CamelFolder *
-mail_tool_get_local_inbox (CamelException *ex)
-{
-	return mail_component_get_local_inbox(mail_component_peek(), ex);
-}
-
 CamelFolder *
 mail_tool_get_inbox (const gchar *url, CamelException *ex)
 {
@@ -103,22 +98,30 @@ mail_tool_get_trash (const gchar *url, int connect, CamelException *ex)
 }
 
 static char *
-mail_tool_get_local_movemail_path (const unsigned char *uri)
+mail_tool_get_local_movemail_path (const unsigned char *uri, CamelException *ex)
 {
 	unsigned char *safe_uri, *c;
-	char *path;
-	
+	char *path, *full;
+	struct stat st;
+
 	safe_uri = g_strdup (uri);
 	for (c = safe_uri; *c; c++)
-		if (strchr ("/:;=|%&#!*^()\\, ", *c) || !isprint ((int) *c))
+		if (strchr("/:;=|%&#!*^()\\, ", *c) || !isprint((int) *c))
 			*c = '_';
+
+	path = g_strdup_printf("%s/mail/spool", mail_component_peek_base_directory(NULL));
+	if (stat(path, &st) == -1 && camel_mkdir(path, 0777) == -1) {
+		camel_exception_setv(ex, CAMEL_EXCEPTION_SYSTEM, _("Could not create spool directory `%s': %s"),
+				     path, g_strerror(errno));
+		g_free(path);
+		return NULL;
+	}
+
+	full = g_strdup_printf("%s/movemail.%s", path, safe_uri);
+	g_free(path);
+	g_free(safe_uri);
 	
-	path = g_strdup_printf ("%s/local/Inbox/movemail.%s",
-				mail_component_peek_base_directory (mail_component_peek ()),
-				safe_uri);
-	g_free (safe_uri);
-	
-	return path;
+	return full;
 }
 
 char *
@@ -133,17 +136,19 @@ mail_tool_do_movemail (const char *source_url, CamelException *ex)
 		return NULL;
 
 	if (strcmp(uri->protocol, "mbox") != 0) {
-		/* FIXME: use right text here post 1.4 */
+		/* This is really only an internal error anyway */
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_URL_INVALID,
-				      _("Could not parse URL `%s'"),
+				      _("Trying to movemail a non-mbox source `%s'"),
 				      source_url);
 		camel_url_free(uri);
 		return NULL;
 	}
 	
 	/* Set up our destination. */
-	dest_path = mail_tool_get_local_movemail_path (source_url);
-	
+	dest_path = mail_tool_get_local_movemail_path (source_url, ex);
+	if (dest_path == NULL)
+		return NULL;
+
 	/* Movemail from source (source_url) to dest_path */
 	camel_movemail (uri->path, dest_path, ex);
 	camel_url_free(uri);
@@ -301,7 +306,8 @@ mail_tool_uri_to_folder (const char *uri, guint32 flags, CamelException *ex)
 	char *curi = NULL;
 
 	g_return_val_if_fail (uri != NULL, NULL);
-	
+
+	/* TODO: vtrash and vjunk are no longer used for these uri's */
 	if (!strncmp (uri, "vtrash:", 7))
 		offset = 7;
 	else if (!strncmp (uri, "vjunk:", 6))
@@ -358,7 +364,6 @@ mail_tool_uri_to_folder (const char *uri, guint32 flags, CamelException *ex)
 	return folder;
 }
 
-
 /**
  * mail_tools_x_evolution_message_parse:
  * @in: GtkSelectionData->data
@@ -400,25 +405,26 @@ mail_tools_x_evolution_message_parse (char *in, unsigned int inlen, GPtrArray **
 	return folder;
 }
 
-
 char *
 mail_tools_folder_to_url (CamelFolder *folder)
 {
-	char *service_url, *url;
-	const char *full_name;
-	CamelService *service;
-	
+	CamelURL *url;
+	char *out;
+
 	g_return_val_if_fail (CAMEL_IS_FOLDER (folder), NULL);
-	
-	full_name = folder->full_name;
-	while (*full_name == '/')
-		full_name++;
-	
-	service = (CamelService *) folder->parent_store;
-	service_url = camel_url_to_string (service->url, CAMEL_URL_HIDE_ALL);
-	url = g_strdup_printf ("%s%s%s", service_url, service_url[strlen (service_url)-1] != '/' ? "/" : "",
-			       full_name);
-	g_free (service_url);
-	
-	return url;
+
+	url = camel_url_copy(((CamelService *)folder->parent_store)->url);
+	if (((CamelService *)folder->parent_store)->provider->url_flags  & CAMEL_URL_FRAGMENT_IS_PATH) {
+		camel_url_set_fragment(url, folder->full_name);
+	} else {
+		char *name = g_alloca(strlen(folder->full_name)+2);
+
+		sprintf(name, "/%s", folder->full_name);
+		camel_url_set_path(url, name);
+	}
+
+	out = camel_url_to_string(url, CAMEL_URL_HIDE_ALL);
+	camel_url_free(url);
+
+	return out;
 }
